﻿using ModelExchanger.AnalysisDataModel;
using ModelExchanger.AnalysisDataModel.Contracts;
using ModelExchanger.AnalysisDataModel.Enums;
using ModelExchanger.AnalysisDataModel.Integration.Bootstrapper;
using ModelExchanger.AnalysisDataModel.Libraries;
using ModelExchanger.AnalysisDataModel.Loads;
using ModelExchanger.AnalysisDataModel.Models;
using ModelExchanger.AnalysisDataModel.StructuralElements;
using ModelExchanger.AnalysisDataModel.Subtypes;
using SCIA.OpenAPI;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using UnitsNet;
using Environment = SCIA.OpenAPI.Environment;

namespace SciaEngineer.OpenApi.SEN22_0
{
    internal class Program
    {
        private static string SenInstallationPath = "C:\\Program Files\\SCIA\\Engineer22.0\\";
        internal static string SenTempFolder = "C:\\Users\\ginov\\ESA22.0\\Temp\\ADMsync\\";
        private static string SenEmptyProject = $"{Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location)}\\EsaProjects\\EmptyProject22.0.esa";

        static void Main(string[] args)
        {
            Tools.InitializeApplication(SenInstallationPath);


            CreateCustomModel();


            Console.WriteLine();
            Console.WriteLine("Finished.");
            Console.WriteLine("Press any key to continue...");
            Console.ReadLine();
        }


        private static void CreateCustomModel()
        {
            // Start SCIA Engineer and open project
            (Environment Environment, EsaProject Project) senData = Tools.StartSciaEngineer(SenInstallationPath, SenTempFolder, SenEmptyProject);

            // Create ADM model (in memory)
            AnalysisModel model = new AnalysisModel();
            AnalysisDataModelBootstrapper admBootstrapper = new AnalysisDataModelBootstrapper();
            using (var scope = admBootstrapper.CreateThreadedScope())
            {
                IAnalysisModelService AnalysisModelService = scope.GetService<IAnalysisModelService>();
                CreateModelUsingADM(model, AnalysisModelService);

                //IAnalysisModelQuery AnalysisModelqueryService = scope.GetService<IAnalysisModelQuery>();
                
                //IStructuralAnalysisModelQuery AnalysisModelQueryService = scope.GetService<IStructuralAnalysisModelQuery>();
                //IReadOnlyList<StructuralCrossSection> crossSections = AnalysisModelQueryService.OfType<StructuralCrossSection>(model);  //model ??
                //IReadOnlyList<StructuralLoadCase> loadCases = AnalysisModelQueryService.OfType<StructuralLoadCase>(model);

                //IAnalysisModelItemDeleteService AnalysisModelItemDeleteService = scope.GetService<IAnalysisModelItemDeleteService>();
                //var beamToDelete = model.OfType<StructuralCurveMember>().FirstOrDefault(x => x.Name == "B1");
                //AnalysisModelItemDeleteService.CascadingRemoveItemFromModel(model, beamToDelete);
            }

            if (model.Count == 0)
            {
                Console.WriteLine("No items have been added to the model!");
                return;
            }

            // Transfer ADM model to a SEN project model (in memory)
            foreach (IAnalysisObject admObject in model)
            {
                senData.Project.Model.CreateAdmObject(admObject);
            }

            // Send the SEN project model to the SCIA Engineer data engine
            senData.Project.Model.RefreshModel_ToSCIAEngineer();

            // Calculate            
            senData.Project.RunCalculation();

            // Get Results
            GetCalculatedResults(senData.Project, model, OutputContent.Extended, OutputLocation.Txt_file);

            // Close project and SCIA Engineer
            Console.WriteLine("Press any key... "); Console.ReadKey();
            senData.Project.CloseProject(SaveMode.SaveChangesNo);
            senData.Environment.Dispose();
        }


        private static void CreateModelUsingADM(AnalysisModel model, IAnalysisModelService modelService)
        {
            /*
            * Use the modelService parameter to add new objects to the model. e.g; modelService.AddItemToModel(model, ...);
            * These objects will be validated and should validation fail, the model.ValidationResults property will hold the validation results.
            */

            gvModels.GV_Example01(ref model, modelService);

            // Make sure the model is valid. Will throw an exception if the model is not valid.
            model.EnforceModelValidity();
        }



        private static void GetCalculatedResults(EsaProject proj, AnalysisModel model, OutputContent oc, OutputLocation ol)
        {
            //Elem_0D
            gvResults.OutputResults(proj, model, oc, ol, ResultsFor.Elem_0D, ResultsType.InternalForces, ResultsCase.Loadcase);

            //Elem_1D
            gvResults.OutputResults(proj, model, oc, ol, ResultsFor.Elem_1D, ResultsType.InternalForces, ResultsCase.Loadcase);
            gvResults.OutputResults(proj, model, oc, ol, ResultsFor.Elem_1D, ResultsType.Deformations, ResultsCase.Loadcase);
            gvResults.OutputResults(proj, model, oc, ol, ResultsFor.Elem_1D, ResultsType.InternalForces, ResultsCase.Combination);
            gvResults.OutputResults(proj, model, oc, ol, ResultsFor.Elem_1D, ResultsType.Deformations, ResultsCase.Combination);

            //Elem_2D
            gvResults.OutputResults(proj, model, oc, ol, ResultsFor.Elem_2D, ResultsType.InternalForces, ResultsCase.Loadcase);
            gvResults.OutputResults(proj, model, oc, ol, ResultsFor.Elem_2D, ResultsType.Deformations, ResultsCase.Loadcase);
            gvResults.OutputResults(proj, model, oc, ol, ResultsFor.Elem_2D, ResultsType.InternalForces, ResultsCase.Combination);
            gvResults.OutputResults(proj, model, oc, ol, ResultsFor.Elem_2D, ResultsType.Deformations, ResultsCase.Combination);
        }
 

    }
}
