﻿using ModelExchanger.AnalysisDataModel.Contracts;
using MADM_Enums = ModelExchanger.AnalysisDataModel.Enums;    //gv modified
using ModelExchanger.AnalysisDataModel.Libraries;
using ModelExchanger.AnalysisDataModel.Loads;
using ModelExchanger.AnalysisDataModel.Models;
using ModelExchanger.AnalysisDataModel.StructuralElements;
using ModelExchanger.AnalysisDataModel.StructuralReferences.Curves;
using ModelExchanger.AnalysisDataModel.Subtypes.Load;
using ModelExchanger.AnalysisDataModel.Subtypes;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnitsNet;

namespace SciaEngineer.OpenApi.SEN22_0
{
    internal class gvModels
    {
        internal static void GV_Example01(ref AnalysisModel model, IAnalysisModelService modelService)
        {
            double o = 0.0;
            double a = 6.0;
            double b = 8.0;
            double c = 3.0;

            // Define
            // --- material
            StructuralMaterial mat_S355 = new StructuralMaterial(Guid.NewGuid(), "mat_S355", MADM_Enums.MaterialType.Steel, "S 355");
            StructuralMaterial mat_C30_37 = new StructuralMaterial(Guid.NewGuid(), "mat_C30/37", MADM_Enums.MaterialType.Concrete, "C30 / 37");
            modelService.AddItemsToModel(model, new[] { mat_S355, mat_C30_37 });


            // Define
            // --- cross-section:  manufactured - steel
            StructuralManufacturedCrossSection css_hea260 = new StructuralManufacturedCrossSection(Guid.NewGuid(), "css_hea260", mat_S355, "HEA260", MADM_Enums.FormCode.ISection, MADM_Enums.DescriptionId.EuropeanIBeam);
            modelService.AddItemsToModel(model, new[] { css_hea260 });


            // Define
            // --- cross-section:  parametric rectangle - concrete
            List<double> dimensions_mm = new List<double> { 200.0, 400.0 };
            List<Length> lst_Lengths = new List<Length>();
            foreach (double dimension in dimensions_mm) { lst_Lengths.Add(Length.FromMillimeters(dimension)); }
            StructuralParametricCrossSection css_R20_40 = new StructuralParametricCrossSection(Guid.NewGuid(), "css_R20/40", mat_C30_37, MADM_Enums.ProfileLibraryId.Rectangle, lst_Lengths.AsReadOnly());
            modelService.AddItemsToModel(model, new[] { css_R20_40 });

            // Define
            // --- node
            StructuralPointConnection n1 = new StructuralPointConnection(Guid.NewGuid(), "n1", Length.FromMeters(o), Length.FromMeters(o), Length.FromMeters(o));
            StructuralPointConnection n2 = new StructuralPointConnection(Guid.NewGuid(), "n2", Length.FromMeters(a), Length.FromMeters(o), Length.FromMeters(o));
            StructuralPointConnection n3 = new StructuralPointConnection(Guid.NewGuid(), "n3", Length.FromMeters(a), Length.FromMeters(b), Length.FromMeters(o));
            StructuralPointConnection n4 = new StructuralPointConnection(Guid.NewGuid(), "n4", Length.FromMeters(o), Length.FromMeters(b), Length.FromMeters(o));
            StructuralPointConnection n11 = new StructuralPointConnection(Guid.NewGuid(), "n11", Length.FromMeters(o), Length.FromMeters(o), Length.FromMeters(c));
            StructuralPointConnection n12 = new StructuralPointConnection(Guid.NewGuid(), "n12", Length.FromMeters(a), Length.FromMeters(o), Length.FromMeters(c));
            StructuralPointConnection n13 = new StructuralPointConnection(Guid.NewGuid(), "n13", Length.FromMeters(a), Length.FromMeters(b), Length.FromMeters(c));
            StructuralPointConnection n14 = new StructuralPointConnection(Guid.NewGuid(), "n14", Length.FromMeters(o), Length.FromMeters(b), Length.FromMeters(c));

            StructuralPointConnection create_node(string name, double x, double y, double z) => new StructuralPointConnection(Guid.NewGuid(), name, Length.FromMeters(x), Length.FromMeters(y), Length.FromMeters(z));
            StructuralPointConnection n21 = create_node("n21", 0.0, 0.0, 6.0);
            StructuralPointConnection n22 = create_node("n22", 6.0, 0.0, 6.0);
            StructuralPointConnection n23 = create_node("n23", 6.0, 8.0, 6.0);
            StructuralPointConnection n24 = create_node("n24", 0.0, 8.0, 6.0);
            modelService.AddItemsToModel(model, new[] { n1, n2, n3, n4, n11, n12, n13, n14, n21, n22, n23, n24 });

            // intermediate definitions:  can be used to set conditions for a nodal supports and for a hinge at a node
            Constraint<ForcePerLength?> displ_fix = new Constraint<ForcePerLength?>(MADM_Enums.ConstraintType.Rigid, null);
            Constraint<ForcePerLength?> displ_free = new Constraint<ForcePerLength?>(MADM_Enums.ConstraintType.Free, null);
            Constraint<ForcePerLength?> displ_flex_001MN = new Constraint<ForcePerLength?>(MADM_Enums.ConstraintType.Flexible, ForcePerLength.FromMeganewtonsPerMeter(1.0));
            Constraint<RotationalStiffness?> rot_fix = new Constraint<RotationalStiffness?>(MADM_Enums.ConstraintType.Rigid, null);
            Constraint<RotationalStiffness?> rot_free = new Constraint<RotationalStiffness?>(MADM_Enums.ConstraintType.Free, null);

            // Define
            // --- support in node
            StructuralPointSupport su1 = new StructuralPointSupport(Guid.NewGuid(), "su1", n1);
            StructuralPointSupport su2 = new StructuralPointSupport(Guid.NewGuid(), "su2", n2);
            StructuralPointSupport su3 = new StructuralPointSupport(Guid.NewGuid(), "su3", n3) { RotationX = rot_free, RotationY = rot_free, RotationZ = rot_fix };
            StructuralPointSupport su4 = new StructuralPointSupport(Guid.NewGuid(), "su4", n4);
            modelService.AddItemsToModel(model, new[] { su1, su2, su3, su4 });

            // Define
            // --- beam
            Curve<StructuralPointConnection> b1_curve = new Curve<StructuralPointConnection>(MADM_Enums.CurveGeometricalShape.Line, new[] { n1, n11 });
            StructuralCurveMember b1 = new StructuralCurveMember(Guid.NewGuid(), "b1", new[] { b1_curve }, css_hea260);
            Curve<StructuralPointConnection> b2_curve = new Curve<StructuralPointConnection>(MADM_Enums.CurveGeometricalShape.Line, new[] { n2, n12 });
            StructuralCurveMember b2 = new StructuralCurveMember(Guid.NewGuid(), "b2", new[] { b2_curve }, css_hea260);
            Curve<StructuralPointConnection> b3_curve = new Curve<StructuralPointConnection>(MADM_Enums.CurveGeometricalShape.Line, new[] { n3, n13 });
            StructuralCurveMember b3 = new StructuralCurveMember(Guid.NewGuid(), "b3", new[] { b3_curve }, css_hea260);
            Curve<StructuralPointConnection> b4_curve = new Curve<StructuralPointConnection>(MADM_Enums.CurveGeometricalShape.Line, new[] { n4, n14 });
            StructuralCurveMember b4 = new StructuralCurveMember(Guid.NewGuid(), "b4", new[] { b4_curve }, css_hea260);
            modelService.AddItemsToModel(model, new[] { b1, b2, b3, b4 });

            // Define
            // --- hinge on beam
            string Constraints = "111100";
            RelConnectsStructuralMember h1 = new RelConnectsStructuralMember(Guid.NewGuid(), "h1", b4)
            {
                Position = MADM_Enums.Position.End,
                TranslationX = Constraints.Substring(0, 1) == "1" ? displ_fix : displ_free,
                TranslationY = Constraints.Substring(1, 1) == "1" ? displ_fix : displ_free,
                TranslationZ = Constraints.Substring(2, 1) == "1" ? displ_fix : displ_free,
                RotationX = Constraints.Substring(3, 1) == "1" ? rot_fix : rot_free,
                RotationY = Constraints.Substring(4, 1) == "1" ? rot_fix : rot_free,
                RotationZ = Constraints.Substring(5, 1) == "1" ? rot_fix : rot_free
            };
            modelService.AddItemsToModel(model, new[] { h1 });

            // Define
            // --- plate, wall
            Curve<StructuralPointConnection> slab1_curve1 = new Curve<StructuralPointConnection>(MADM_Enums.CurveGeometricalShape.Line, new[] { n11, n12 });
            Curve<StructuralPointConnection> slab1_curve2 = new Curve<StructuralPointConnection>(MADM_Enums.CurveGeometricalShape.Line, new[] { n12, n13 });
            Curve<StructuralPointConnection> slab1_curve3 = new Curve<StructuralPointConnection>(MADM_Enums.CurveGeometricalShape.Line, new[] { n13, n14 });
            Curve<StructuralPointConnection> slab1_curve4 = new Curve<StructuralPointConnection>(MADM_Enums.CurveGeometricalShape.Line, new[] { n14, n11 });
            StructuralSurfaceMember slab1 = new StructuralSurfaceMember(Guid.NewGuid(), "slab1", new[] { slab1_curve1, slab1_curve2, slab1_curve3, slab1_curve4 }, mat_C30_37, Length.FromMillimeters(314));

            Curve<StructuralPointConnection> wall1_curve1 = new Curve<StructuralPointConnection>(MADM_Enums.CurveGeometricalShape.Line, new[] { n11, n21 });
            Curve<StructuralPointConnection> wall1_curve2 = new Curve<StructuralPointConnection>(MADM_Enums.CurveGeometricalShape.Line, new[] { n21, n22 });
            Curve<StructuralPointConnection> wall1_curve3 = new Curve<StructuralPointConnection>(MADM_Enums.CurveGeometricalShape.Line, new[] { n22, n12 });
            Curve<StructuralPointConnection> wall1_curve4 = new Curve<StructuralPointConnection>(MADM_Enums.CurveGeometricalShape.Line, new[] { n12, n11 });
            StructuralSurfaceMember wall1 = new StructuralSurfaceMember(Guid.NewGuid(), "wall1", new[] { wall1_curve1, wall1_curve2, wall1_curve3, wall1_curve4 }, mat_C30_37, Length.FromMillimeters(250));

            Curve<StructuralPointConnection> wall2_curve1 = new Curve<StructuralPointConnection>(MADM_Enums.CurveGeometricalShape.Line, new[] { n13, n23 });
            Curve<StructuralPointConnection> wall2_curve2 = new Curve<StructuralPointConnection>(MADM_Enums.CurveGeometricalShape.Line, new[] { n23, n24 });
            Curve<StructuralPointConnection> wall2_curve3 = new Curve<StructuralPointConnection>(MADM_Enums.CurveGeometricalShape.Line, new[] { n24, n14 });
            Curve<StructuralPointConnection> wall2_curve4 = new Curve<StructuralPointConnection>(MADM_Enums.CurveGeometricalShape.Line, new[] { n14, n13 });
            StructuralSurfaceMember wall2 = new StructuralSurfaceMember(Guid.NewGuid(), "wall2", new[] { wall2_curve1, wall2_curve2, wall2_curve3, wall2_curve4 }, mat_C30_37, Length.FromMillimeters(250));

            Curve<StructuralPointConnection> slab2_curve1 = new Curve<StructuralPointConnection>(MADM_Enums.CurveGeometricalShape.Line, new[] { n21, n22 });
            Curve<StructuralPointConnection> slab2_curve2 = new Curve<StructuralPointConnection>(MADM_Enums.CurveGeometricalShape.Line, new[] { n22, n23 });
            Curve<StructuralPointConnection> slab2_curve3 = new Curve<StructuralPointConnection>(MADM_Enums.CurveGeometricalShape.Line, new[] { n23, n24 });
            Curve<StructuralPointConnection> slab2_curve4 = new Curve<StructuralPointConnection>(MADM_Enums.CurveGeometricalShape.Line, new[] { n24, n21 });
            StructuralSurfaceMember slab2 = new StructuralSurfaceMember(Guid.NewGuid(), "slab2", new[] { slab2_curve1, slab2_curve2, slab2_curve3, slab2_curve4 }, mat_C30_37, Length.FromMillimeters(250));

            modelService.AddItemsToModel(model, new[] { slab1, wall1, wall2, slab2 });


            // Define 
            // --- load group
            StructuralLoadGroup lgPerm = new StructuralLoadGroup(Guid.NewGuid(), "lgPerm", MADM_Enums.LoadGroupType.Permanent);
            StructuralLoadGroup lgVar = new StructuralLoadGroup(Guid.NewGuid(), "lgVar", MADM_Enums.LoadGroupType.Variable)
            {
                Load = MADM_Enums.Load.Domestic,
                Relation = MADM_Enums.Relation.Standard,
            };
            StructuralLoadGroup lgWind = new StructuralLoadGroup(Guid.NewGuid(), "lgWind", MADM_Enums.LoadGroupType.Variable)
            {
                Load = MADM_Enums.Load.Wind,
                Relation = MADM_Enums.Relation.Exclusive
            };
            modelService.AddItemsToModel(model, new[] { lgPerm, lgVar, lgWind });

            // Define 
            // --- load case
            StructuralLoadCase sw = new StructuralLoadCase(Guid.NewGuid(), "sw", MADM_Enums.ActionType.Permanent, lgPerm, MADM_Enums.LoadCaseType.SelfWeight);
            StructuralLoadCase lc2 = new StructuralLoadCase(Guid.NewGuid(), "lc2", MADM_Enums.ActionType.Permanent, lgPerm, MADM_Enums.LoadCaseType.Standard);
            StructuralLoadCase lc3a = new StructuralLoadCase(Guid.NewGuid(), "lc3a", MADM_Enums.ActionType.Variable, lgVar, MADM_Enums.LoadCaseType.Static)
            {
                Description = "var 1",
                Duration = MADM_Enums.Duration.Long
            };
            StructuralLoadCase lc3b = new StructuralLoadCase(Guid.NewGuid(), "lc3b", MADM_Enums.ActionType.Variable, lgVar, MADM_Enums.LoadCaseType.Static)
            {
                Description = "var 2",
                Duration = MADM_Enums.Duration.Long
            };
            StructuralLoadCase lc4a = new StructuralLoadCase(Guid.NewGuid(), "lc4a", MADM_Enums.ActionType.Variable, lgWind, MADM_Enums.LoadCaseType.Static)
            {
                Description = "wind +",
                Duration = MADM_Enums.Duration.Short
            };
            StructuralLoadCase lc4b = new StructuralLoadCase(Guid.NewGuid(), "lc4b", MADM_Enums.ActionType.Variable, lgWind, MADM_Enums.LoadCaseType.Static)
            {
                Description = "wind -",
                Duration = MADM_Enums.Duration.Short
            };

            modelService.AddItemsToModel(model, new[] { sw, lc2, lc3a, lc3b, lc4a, lc4b });


            // Define 
            // --- combination
            List<StructuralLoadCombinationData> combidatas = new List<StructuralLoadCombinationData>
            {
                new StructuralLoadCombinationData(sw, 1.00),
                new StructuralLoadCombinationData(lc2, 1.00),
                new StructuralLoadCombinationData(lc3a, 1.00), new StructuralLoadCombinationData(lc3b, 1.00),
                new StructuralLoadCombinationData(lc4a, 1.00), new StructuralLoadCombinationData(lc4b, 1.00)
            };
            StructuralLoadCombination coLin = new StructuralLoadCombination(Guid.NewGuid(), "coLin", MADM_Enums.LoadCaseCombinationCategory.UltimateLimitState, combidatas.AsReadOnly());
            StructuralLoadCombination coENULSb = new StructuralLoadCombination(Guid.NewGuid(), "coENULSb", MADM_Enums.LoadCaseCombinationCategory.AccordingNationalStandard, combidatas.AsReadOnly())
            {
                NationalStandard = MADM_Enums.LoadCaseCombinationStandard.EnUlsSetB
            };
            modelService.AddItemsToModel(model, new[] { coLin, coENULSb });

            // Define 
            // --- load -- surface load on 2D member
            StructuralSurfaceAction sf1 = new StructuralSurfaceAction(Guid.NewGuid(), "sf1", Pressure.FromKilonewtonsPerSquareMeter(-31.4), slab1, lc2)
            {
                Direction = MADM_Enums.ActionDirection.Z,
                CoordinateSystem = MADM_Enums.CoordinateSystem.Global
            };
            StructuralSurfaceAction sf4a = new StructuralSurfaceAction(Guid.NewGuid(), "sf4a", Pressure.FromKilonewtonsPerSquareMeter(10.0), wall1, lc4a)
            {
                Direction = MADM_Enums.ActionDirection.Y,
                CoordinateSystem = MADM_Enums.CoordinateSystem.Global
            };
            StructuralSurfaceAction sf4b = new StructuralSurfaceAction(Guid.NewGuid(), "sf4b", Pressure.FromKilonewtonsPerSquareMeter(-10.0), wall2, lc4b)
            {
                Direction = MADM_Enums.ActionDirection.Y,
                CoordinateSystem = MADM_Enums.CoordinateSystem.Global
            };
            modelService.AddItemsToModel(model, new[] { sf1, sf4a, sf4b });

            // Define 
            // --- load -- temperature load on 2D member
            StructuralSurfaceActionThermal sf2 = new StructuralSurfaceActionThermal(Guid.NewGuid(), "sf2", slab1, lc3a, Temperature.FromKelvins(80.0));
            modelService.AddItemsToModel(model, new[] { sf2 });

            // Define 
            // --- load -- free load on 2D member
            Coordinate create_coord(string name, double x, double y, double z) => new Coordinate(name, Length.FromMeters(x), Length.FromMeters(y), Length.FromMeters(z));
            Coordinate coord1 = create_coord("ni1", 1.0, 1.0, 6.0);
            Coordinate coord2 = create_coord("ni2", 1.0, 7.0, 6.0);
            Coordinate coord3 = create_coord("ni3", 5.0, 7.0, 6.0);
            Coordinate coord4 = create_coord("ni4", 5.0, 1.0, 6.0);
            Curve<Coordinate> freeload_curve1 = new Curve<Coordinate>(MADM_Enums.CurveGeometricalShape.Line, new[] { coord1, coord2 });
            Curve<Coordinate> freeload_curve2 = new Curve<Coordinate>(MADM_Enums.CurveGeometricalShape.Line, new[] { coord2, coord3 });
            Curve<Coordinate> freeload_curve3 = new Curve<Coordinate>(MADM_Enums.CurveGeometricalShape.Line, new[] { coord3, coord4 });
            Curve<Coordinate> freeload_curve4 = new Curve<Coordinate>(MADM_Enums.CurveGeometricalShape.Line, new[] { coord4, coord1 });
            List<Curve<Coordinate>> freeload_area = new List<Curve<Coordinate>> { freeload_curve1, freeload_curve2, freeload_curve3, freeload_curve4 };
            StructuralSurfaceActionFree sf3 = new StructuralSurfaceActionFree(Guid.NewGuid(), "sf3", lc3b, freeload_area.AsReadOnly(), new StructuralSurfaceActionFreeLoadUniform(Pressure.FromKilonewtonsPerSquareMeter(-50.0)))
            {
                Direction = MADM_Enums.ActionDirection.Z,
                Validity = MADM_Enums.Validity.FromTo,
                ValidityFrom = Length.FromMeters(5.9),
                ValidityTo = Length.FromMeters(6.1),
                Location = MADM_Enums.Location.Projection
            };
            modelService.AddItemsToModel(model, new[] { sf3 });

        }


        internal static void GV_Example02(ref AnalysisModel model, IAnalysisModelService modelService)
        {
            // Define
            // --- material
            StructuralMaterial mat_S355 = new StructuralMaterial(Guid.NewGuid(), "mat_S355", MADM_Enums.MaterialType.Steel, "S 355");
            modelService.AddItemsToModel(model, new[] { mat_S355 });


            // Define
            // --- cross-section:  manufactured - steel
            StructuralManufacturedCrossSection css_hea260 = new StructuralManufacturedCrossSection(Guid.NewGuid(), "css_hea260", mat_S355, "HEA260", MADM_Enums.FormCode.ISection, MADM_Enums.DescriptionId.EuropeanIBeam);
            modelService.AddItemsToModel(model, new[] { css_hea260 });

            // Define
            // --- node
            StructuralPointConnection create_node(string name, double x, double y, double z) => new StructuralPointConnection(Guid.NewGuid(), name, Length.FromMeters(x), Length.FromMeters(y), Length.FromMeters(z));
            StructuralPointConnection n1 = create_node("n1", 0.0, 0.0, 0.0);
            StructuralPointConnection n2 = create_node("n2", 10.0, 0.0, 0.0);
            modelService.AddItemsToModel(model, new[] { n1, n2 });

            // intermediate definitions:  can be used to set conditions for a nodal supports and for a hinge at a node
            Constraint<ForcePerLength?> displ_fix = new Constraint<ForcePerLength?>(MADM_Enums.ConstraintType.Rigid, null);
            Constraint<ForcePerLength?> displ_free = new Constraint<ForcePerLength?>(MADM_Enums.ConstraintType.Free, null);
            Constraint<ForcePerLength?> displ_flex_001MN = new Constraint<ForcePerLength?>(MADM_Enums.ConstraintType.Flexible, ForcePerLength.FromMeganewtonsPerMeter(1.0));
            Constraint<RotationalStiffness?> rot_fix = new Constraint<RotationalStiffness?>(MADM_Enums.ConstraintType.Rigid, null);
            Constraint<RotationalStiffness?> rot_free = new Constraint<RotationalStiffness?>(MADM_Enums.ConstraintType.Free, null);

            // Define
            // --- support in node
            StructuralPointSupport su1 = new StructuralPointSupport(Guid.NewGuid(), "su1", n1);
            StructuralPointSupport su2 = new StructuralPointSupport(Guid.NewGuid(), "su2", n2);
            modelService.AddItemsToModel(model, new[] { su1, su2 });

            // Define
            // --- beam
            Curve<StructuralPointConnection> b1_curve = new Curve<StructuralPointConnection>(MADM_Enums.CurveGeometricalShape.Line, new[] { n1, n2 });
            StructuralCurveMember b1 = new StructuralCurveMember(Guid.NewGuid(), "b1", new[] { b1_curve }, css_hea260);
            modelService.AddItemsToModel(model, new[] { b1 });

            // Define 
            // --- load group
            StructuralLoadGroup lgPerm = new StructuralLoadGroup(Guid.NewGuid(), "lgPerm", MADM_Enums.LoadGroupType.Permanent);
            modelService.AddItemsToModel(model, new[] { lgPerm });

            // Define 
            // --- load case
            StructuralLoadCase sw = new StructuralLoadCase(Guid.NewGuid(), "sw", MADM_Enums.ActionType.Permanent, lgPerm, MADM_Enums.LoadCaseType.SelfWeight);
            StructuralLoadCase lc2 = new StructuralLoadCase(Guid.NewGuid(), "lc2", MADM_Enums.ActionType.Permanent, lgPerm, MADM_Enums.LoadCaseType.Standard);
            modelService.AddItemsToModel(model, new[] { sw, lc2 });

            // Define 
            // --- combination
            List<StructuralLoadCombinationData> combidatas = new List<StructuralLoadCombinationData>
            {
                new StructuralLoadCombinationData(sw, 1.00),
                new StructuralLoadCombinationData(lc2, 1.00),
            };
            StructuralLoadCombination coLin = new StructuralLoadCombination(Guid.NewGuid(), "coLin", MADM_Enums.LoadCaseCombinationCategory.UltimateLimitState, combidatas.AsReadOnly());
            StructuralLoadCombination coENULSb = new StructuralLoadCombination(Guid.NewGuid(), "coENULSb", MADM_Enums.LoadCaseCombinationCategory.AccordingNationalStandard, combidatas.AsReadOnly())
            {
                NationalStandard = MADM_Enums.LoadCaseCombinationStandard.EnUlsSetB
            };
            modelService.AddItemsToModel(model, new[] { coLin, coENULSb });

            // Define
            // --- Point load on Elem1D
            StructuralCurveAction<CurveStructuralReferenceOnBeam> lf1 = new StructuralCurveAction<CurveStructuralReferenceOnBeam>(Guid.NewGuid(), "lf1", MADM_Enums.CurveForceAction.OnBeam, ForcePerLength.FromKilonewtonsPerMeter(-20.0), lc2, new CurveStructuralReferenceOnBeam(b1))
            {
                Direction = MADM_Enums.ActionDirection.Z,
                CoordinateDefinition = MADM_Enums.CoordinateDefinition.Relative,
                StartPointRelative = 0.0,
                EndPointRelative = 1.0
            };
            modelService.AddItemsToModel(model, new[] { lf1 });

        }





    }
}
